"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.flattenHierarchies = flattenHierarchies;

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

/* If there is a property label lower in the hierarchy,
do not overwrite it from parent with same name */
function existsInRefs(context, propertySpecs, parentRef) {
  return propertySpecs.some(function (ref) {
    var node = context.propertyCache[ref.ref];
    var parentNode = context.propertyCache[parentRef.ref];
    return node.label === parentNode.label;
  });
}

function flattenHierarchies(context) {
  Object.keys(context.classCache).forEach(function (key) {
    var classNode = context.classCache[key];
    var excluded = [];

    var recurseNode = function recurseNode(node) {
      if (node.subClassOf) {
        var parent = context.classCache[node.subClassOf];
        parent.propertySpecs.forEach(function (parentRef) {
          if (node.excludeParentProperties) {
            excluded.push.apply(excluded, _toConsumableArray(node.excludeParentProperties));
          }
          var exists = existsInRefs(context, classNode.propertySpecs, parentRef);
          if (!exists) {
            classNode.propertySpecs.push(parentRef);
          }
        });
        recurseNode(parent);
      }
    };
    recurseNode(classNode);
    classNode.propertySpecs = classNode.propertySpecs.filter(function (spec) {
      return excluded.indexOf(spec.ref) === -1;
    });
  });
}