'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports._generateGoClass = _generateGoClass;
exports.generateFromClass = generateFromClass;

var _values = require('lodash/values');

var _values2 = _interopRequireDefault(_values);

var _flatten = require('./flatten');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var getTab = function getTab(num) {
  var result = '';
  for (var i = 0; i < num; i++) {
    result += '  ';
  }
  return result;
};

function _fromLinkedClassType(context, ref) {
  var classItem = context.classCache[ref];
  return classItem.label;
}

function _getType(context, property, depth) {
  var range = property.range;


  switch (range.type) {
    case 'Boolean':
      return 'bool';
    case 'Enum':
    case 'Text':
      return 'string';
    case 'Date':
      context.dateUsed = true;
      return 'time.Time';
    case 'Number':
      switch (range.format) {
        case 'Int':
          return 'int';
        case 'Int8':
          return 'int8';
        case 'Int16':
          return 'int16';
        case 'Int32':
          return 'int32';
        case 'Int64':
          return 'int64';
        case 'Float32':
          return 'float32';
        case 'Float64':
          return 'float64';
        default:
          return 'float32';
      }
    case 'NestedObject':
      return property.label;
    case 'LinkedClass':
      return _fromLinkedClassType(context, range.ref);
    default:
      throw new Error('Not expecting type ' + range.type);
  }
}

function _generateGoClass(context, label, propertySpecs) {
  if (context.structs[label]) {
    return;
  }
  context.structs[label] = true;

  var text = 'type ' + label + ' struct {\n';

  propertySpecs.forEach(function (propertySpec) {
    var property = context.propertyCache[propertySpec.ref];
    if (property.range.type === 'NestedObject') {
      context.structs[property.label] = _generateGoClass(context, property.label, property.range.propertySpecs);
    }
    var type = _getType(context, property);
    text += '' + getTab(1) + property.label + ' ' + type + '\n';
  });

  text += '}';

  context.structs[label] = text;

  return text;
}

function generateFromClass(graph, classUid) {
  var opts = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

  /* Create a simple context obj to thread through */
  var context = {
    classCache: {},
    propertyCache: {},
    structs: {},
    dateUsed: false
    // enums: {}
  };

  /* Create a dict lookup for classes and properties for speed and convenience */
  graph.forEach(function (node) {
    if (node.type === 'Class') {
      context.classCache[node.uid] = node;
    } else if (node.type === 'Property') {
      context.propertyCache[node.uid] = node;
    }
  });

  (0, _flatten.flattenHierarchies)(context);

  var currentClass = context.classCache[classUid];
  if (!currentClass) {
    throw new Error('Could not find class ' + classUid + ' in graph');
  }

  _generateGoClass(context, currentClass.label, currentClass.propertySpecs);

  var text = (0, _values2.default)(context.structs).join('\n\n');

  if (context.dateUsed) {
    text = 'import time\n\n' + text;
  }

  return text;
}